/*
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o File        o I2c.c                                           
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
o Author      o EM Microelectronic-Marin SA
o Company     o EM Microelectronic-Marin SA                                                                            
o Project     o i2C                                      
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo                                                                                                                                                                      
o Comments    o I2c - C file
o             o
o             o                                     
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Version     o 1.0
o Date        o 10.03.06
o Change      o Initial                                          
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
*/

// Definitions section
#ifndef __I2c_c__
#define __I2c_c__
// Include section
#include "Header.h"

// usefull I2C variables
unsigned char              VarNbByteI2C = 0;
unsigned char              VarNbBitI2C = 0;
unsigned char              VarShiftRegI2C = 0;
unsigned char              VarStatusRegI2C = 0;

// Function declarations

/*
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Function name o WaitTimeI2C
o Comments      o This function generate a delay time based on Nop
o               o operation. The parameter delay_size introduce the
o               o number of occurences.
o               o Delay = (10xFcpu + (6*Fcpu)*delay_size)
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Change        o Initial                                          
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
*/
void WaitTimeI2C(UINT8 delay_size)
{ 
  while (delay_size != 0)
  {
    Nop;
    delay_size--;
  }
}

/*
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Function name o InitPortI2C
o Comments      o This function set PB6 and PB7 in a compatible I2C
o               o configuration: opendrain + pull-up + default input 
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Change        o Initial                                          
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
*/
void InitPortI2C(void)
{ 
  RegOpenDrainI2C |= (SdaI2C | SclI2C);   // active open drain
  ReleaseI2C;      // input mode
  RegPullDownI2C &= ~(SdaI2C | SclI2C);   // no pull-down
  RegPullUpI2C = (SdaI2C | SclI2C);       // pull-up
  RegOutI2C &= ~(SdaI2C | SclI2C);        // 0 as default output reg   
}

/*
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Function name o StartConditionI2C
o Comments      o This function generate a start condition
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Change        o Initial                                          
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
*/
void StartConditionI2C(void)
{ 
  ReleaseI2C;   // input mode to force high-level
  WaitTimeI2C(10);
  ForceZeroSdaI2C; // low transition on SdaI2C
  WaitTimeI2C(4);
  ForceZeroSclI2C; // force to low SclI2C  
  WaitTimeI2C(6);  
}

/*
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Function name o StopConditionI2C
o Comments      o This function generate a start condition
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Change        o Initial                                          
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
*/
void StopConditionI2C(void)
{ 
  ForceZeroI2C;   // force low level
  WaitTimeI2C(10);
  ReleaseSclI2C; // force SclI2C high
  WaitTimeI2C(4);
  ReleaseSdaI2C; // force SdaI2C high  
  WaitTimeI2C(6);  
}

/*
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Function name o CheckSlaveAcknowledgeI2C
o Comments      o This function wait and check the acknowledge 
o               o generated by slave.
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Change        o Initial                                          
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
*/
void CheckSlaveAcknowledgeI2C(void)
{ 
  unsigned char i;
  ForceZeroSclI2C;  // force low level on SclI2C
  ReleaseSdaI2C; // release SdaI2C to let the slave drive 0
  for (i=0; i<8; i++)     // first step of test for acknowledge
    {    
      WaitTimeI2C(10);
      CheckComingAckI2C;
    }
  ReleaseSclI2C; // start generating clock pulse
  for (i=0; i<3; i++)     // second step of test for acknowledge
    {    
      WaitTimeI2C(1);
      CheckComingAckI2C;
    }
  ForceZeroSclI2C; // end generating clock pulse
  SetAckStatusI2C;       // register the acknowledge in status register   
  WaitTimeI2C(1);
  ForceZeroSdaI2C; // force low level on Sda  
}

/*
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Function name o MasterAcknowledgeI2C
o Comments      o This function generate master acknowledge
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Change        o Initial                                          
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
*/
void MasterAcknowledgeI2C(void)
{ 
    ForceZeroSdaI2C; // force low level on Sda
    WaitTimeI2C(10);
    ReleaseSclI2C; // start generating clock pulse
    WaitTimeI2C(10);
    ForceZeroSclI2C;  // force low level on SclI2C
    WaitTimeI2C(5);
    ReleaseSdaI2C; // release SdaI2C
    SetAckStatusI2C;  // register the acknowledge in status register
}
/*
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Function name o NoMasterAcknowledgeI2C
o Comments      o This function does not generate master acknowledge
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Change        o Initial                                          
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
*/
void NoMasterAcknowledgeI2C(void)
{ 
  ClrAckStatusI2C; // unregister the acknowledge in status register
  ReleaseSdaI2C; // release SdaI2C to go high
  ForceZeroSclI2C;  // force low level on SclI2C  
  WaitTimeI2C(10);
  ReleaseSclI2C; // start generating clock pulse 
  WaitTimeI2C(10);  
  ForceZeroSclI2C;  // force low level on SclI2C
  WaitTimeI2C(5); 
  ClrAckStatusI2C; // unregister the acknowledge in status register
}

/*
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Function name o SendBitI2C
o Comments      o This function send one bit
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Change        o Initial                                          
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
*/
void SendBitI2C(void)
{ 
  ForceZeroSclI2C; // force low level on SclI2C
  if (!CheckErrorStatusI2C) // no error
    {
     if (TestMsbTxI2C) // Msb in shift reg is '1'
      ReleaseSdaI2C; // release SdaI2C to go high        
     else
      ForceZeroSdaI2C; // force low level on Sda
     ShiftLeftVarShiftRegI2C; // shift the Tx buffer
     ReleaseSclI2C; // release SclI2C to go high
     WaitTimeI2C(10);  
   }
}  

/*
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Function name o GetBitI2C
o Comments      o This function get one bit
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Change        o Initial                                          
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
*/
void GetBitI2C(void)
{ 
  if (!CheckErrorStatusI2C) // no error
    {
      ShiftLeftVarShiftRegI2C;  // shift the Rx buffer
      ReleaseSclI2C; // release SclI2C to go high
      while (!TestSclI2CHigh) {}; // Wait until SclI2C fully high
      if (TestSdaI2CHigh)
        LoadOneVarShiftRegI2C;
      else
        LoadZeroVarShiftRegI2C;  
      WaitTimeI2C(10);   
      ForceZeroSclI2C; // force low level on SclI2C
   }
}

/*
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Function name o SendBufferI2C
o Comments      o This function send the buffer content
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Change        o Initial                                          
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
*/
unsigned char SendBufferI2C(unsigned char* pBufferOut, unsigned char BufferDepth, unsigned char DeviceTypeI2C)
{ 
  unsigned char MasterRead;
  InitPortI2C();
  StartConditionI2C();
  for (VarNbByteI2C = BufferDepth-1; VarNbByteI2C!= 0xFF; VarNbByteI2C--)
    {
      MasterRead = NO;
      if ((VarNbByteI2C == BufferDepth-3) && (CheckRxModeStatusI2C) && (CheckDeviceMemoryI2C))
        {// in case of reading memory access the Memory adress must be put again after the 
          StartConditionI2C();// header
          VarNbByteI2C = BufferDepth-1; // -> resend the Memory device adress
          MasterRead = YES;   
        }
      VarNbBitI2C = 0;
      VarShiftRegI2C = pBufferOut[VarNbByteI2C]; // load shift reg. TX with the buffer
      if (MasterRead == YES) VarShiftRegI2C |= ReadSlaveI2C; // in this case force R_nW = Read !
      while (VarNbBitI2C < 8) // Send the 8 bits
        {
          SendBitI2C();
          VarNbBitI2C++; 
        }
      CheckSlaveAcknowledgeI2C();
      if (MasterRead == YES) return VarStatusRegI2C; // Receive coming byte ... from GetBufferI2C      
    }  
    StopConditionI2C();
    return VarStatusRegI2C;
}

/*
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Function name o GetBufferI2C
o Comments      o This function load the buffer with the coming byte
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Change        o Initial                                          
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
*/
unsigned char GetBufferI2C(unsigned char* pBufferIn, unsigned char BufferDepth)
{ 
  InitPortI2C();
  StartConditionI2C();
  for (VarNbByteI2C = BufferDepth-1; VarNbByteI2C!=0xFF; VarNbByteI2C--)
    {
      SetRxModeStatusI2C;
      SendBufferI2C(pBufferIn, BufferDepth, DeviceMemoryI2C); // send header
      if (CheckErrorStatusI2C) return VarStatusRegI2C;; // if error return to main program !
      for (VarNbByteI2C = BufferDepth-3; VarNbByteI2C!= 0xFF; VarNbByteI2C--) // header already sent
        {
          VarNbBitI2C = 0;
          VarShiftRegI2C = 0;
          ReleaseSdaI2C;
          while (VarNbBitI2C < 8) // Send the 8 bits
            {
              GetBitI2C();
              VarNbBitI2C++; 
            }
          if (VarNbByteI2C == 0x00) 
            {
              NoMasterAcknowledgeI2C();
              StopConditionI2C();
              pBufferIn[VarNbByteI2C] = VarShiftRegI2C;              
              return VarStatusRegI2C;
            }
          else 
            {
              MasterAcknowledgeI2C();
              pBufferIn[VarNbByteI2C] = VarShiftRegI2C;
            }
        }
    }
  return VarStatusRegI2C;
}


#endif /* __I2c_c__ */

